#
# This script was written by George A. Theall, <theall@tifaware.com>.
#
# See the Nessus Scripts License for details.
#

if (description) {
  script_id(12643);
  script_version ("$Revision: 1.1 $");
 
  name["english"] = "IMP Detection";
  script_name(english:name["english"]);
 
  desc["english"] = "
This script detects whether the remote host is running IMP and extracts
its version number if so.  IMP is a PHP-based webmail package from The
Horde Project that provides access to mail accounts via POP3 or IMAP. 

Risk factor : None";
  script_description(english:desc["english"]);
 
  summary["english"] = "Checks for the presence of IMP";
  script_summary(english:summary["english"]);
 
  script_category(ACT_GATHER_INFO);
  script_copyright(english:"This script is Copyright (C) 2004 George A. Theall");

  family["english"] = "General";
  script_family(english:family["english"]);

  script_dependencie("find_service.nes");
  script_require_ports("Services/www", 80);

  exit(0);
}

include("global_settings.inc");
include("http_func.inc");
include("http_keepalive.inc");

debug_level = 0;

port = get_http_port(default:80);
if (!get_port_state(port)) exit(0);

if (!can_host_php(port:port)) exit(0);

# Search for IMP in a couple of different locations.
# nb: "webmail" is not a standard but is commonly used.
dirs = make_list("/", "/imp", "/horde/imp", "/webmail", cgi_dirs());
foreach d (dirs) {
  # Search in a couple of different pages.
  files = make_list(
    "/services/help/?module=imp&show=about",
    "/docs/CHANGES", "/test.php", "/README", "/lib/version.phps",
    "/status.php3"
  );
  foreach f (files) {
    if (debug_level) display("debug: testing ", d, f, " on port ", port, "...\n");

    # Get the page.
    req = http_get(item:string(d, f), port:port);
    res = http_keepalive_send_recv(port:port, data:req);
    if (res == NULL) exit(0);           # can't connect
    if (debug_level) display("debug: res =>>", res, "<<\n");
    if (egrep(pattern:"200 OK", string:res, icase:TRUE)) {
      # Specify pattern used to identify version string.
      # - version 4.x
      if (f =~ "^/services/help") {
        pat = ">This is Imp (.+)\.<";
      }
      # - version 3.x
      else if (f == "/docs/CHANGES") {
        pat = "^ *v(.+) *$";
      }
      #   nb: test.php available is itself a vulnerability but sometimes available.
      else if (f == "/test.php") {
        pat = "^ *<li>IMP: +(.+) *</li> *$";
      }
      #   nb: README is not guaranteed to be either available or accurate!!!
      else if (f == "/README") {
        pat = "^Version +(.+) *$";
      }
      #   nb: another security risk -- ability to view PHP source.
      else if (f == "/lib/version.phps") {
        pat = "IMP_VERSION', '(.+)'";
      }
      # - version 2.x
      else if (f == "/status.php3") {
        pat = ">IMP, Version (.+)<";
      }
      # - someone updated files but forgot to add a pattern???
      else {
        display("Don't know how to handle file '", f, "'!\n");
        exit(1);
      }

      # Get the version string.
      if (debug_level) display("debug: grepping results for =>>", pat, "<<\n");
      matches = egrep(pattern:pat, string:res);
      foreach match (split(matches)) {
        match = chomp(match);
        if (debug_level) display("debug: grepping >>", match, "<< for =>>", pat, "<<\n");
        vers = eregmatch(pattern:pat, string:match);
        if (vers == NULL) break;

        if (debug_level) display("debug: IMP version =>>", vers[1], "<<\n");
        set_kb_item(name:string("www/", port, "/imp/version"), value:vers[1]);

        report = "The remote host is running IMP " + vers[1] + " under " + d + ".\n" +
                 "IMP is a PHP-based webmail package from The Horde Project that\n" +
                 "provides access to mail accounts via POP3 or IMAP.\n";
        security_note(port:port, data:report);

        # No sense testing any further since we got the version number.
        exit(0);
      }
    }
  }
}
