#
# This script was written by George A. Theall, <theall@tifaware.com>.
#
# See the Nessus Scripts License for details.
#

if (description) {
  script_id(12644);
  script_version ("$Revision: 1.2 $");
 
  name["english"] = "Open WebMail Detection";
  script_name(english:name["english"]);
 
  desc["english"] = "
This script detects whether the remote host is running Open WebMail and
extracts its version number if so.  Open WebMail is a webmail package
written in Perl that provides access to mail accounts via POP3 or IMAP. 

Risk factor : None";
  script_description(english:desc["english"]);
 
  summary["english"] = "Checks for the presence of Open WebMail";
  script_summary(english:summary["english"]);
 
  script_category(ACT_GATHER_INFO);
  script_copyright(english:"This script is Copyright (C) 2004 George A. Theall");

  family["english"] = "General";
  script_family(english:family["english"]);

  script_dependencie("http_version.nasl");
  script_require_ports("Services/www", 80);

  exit(0);
}

include("global_settings.inc");
include("http_func.inc");
include("http_keepalive.inc");

port = get_http_port(default:80);
if (!get_port_state(port)) exit(0);

# Search for Open Webmail in a couple of different locations.
dirs = make_list("", "/openwebmail", cgi_dirs());
foreach d (dirs) {
  # Search in a couple of different pages.
  files = make_list("/", "/openwebmail.pl", "/doc/changes.txt");
  foreach f (files) {
    if (debug_level) display("debug: testing ", d, f, " on port ", port, "...\n");

    # Get the page.
    req = http_get(item:string(d, f), port:port);
    res = http_keepalive_send_recv(port:port, data:req);
    if (res == NULL) exit(0);           # can't connect
    if (debug_level) display("debug: res =>>", res, "<<\n");
    # nb: make sure it uses the Open WebMail title.
    if ("200 OK" >< res && res =~ "TITLE>.*Open WebMail") {
      # Specify pattern used to identify version string.
      if (f =~ "/" || f =~ "openwebmail\.pl$") {
        pat = "version (.+)$";
      }
      else if (f =~ "/changes\.txt$") {
        # "mm/dd/yyyy (version x.yy)"
        pat = "[0-1][1-9]/[0-3][0-9]/20[0-9][0-9] .version (.+)$";
      }
      # - someone updated files but forgot to add a pattern???
      else {
        display("Don't know how to handle file '", f, "'!\n");
        exit(1);
      }

      # Get the version string.
      if (debug_level) display("debug: grepping results for =>>", pat, "<<\n");
      matches = egrep(pattern:pat, string:res);
      foreach match (split(matches)) {
        match = chomp(match);
        if (debug_level) display("debug: grepping >>", match, "<< for =>>", pat, "<<\n");
        vers = eregmatch(pattern:pat, string:match);
        if (vers == NULL) break;
        if (debug_level) display("debug: version =>>", vers[1], "<<\n");

        set_kb_item(name:string("www/", port, "/openwebmail/version"), value:vers[1]);

        report = "The remote host is running Open WebMail " + vers[1] + " under " + d + ".\n" +
                 "Open WebMail is a webmail package written in Perl that provides\n" +
                 "access to mail accounts via POP3 or IMAP.\n";
        security_note(port:port, data:report);

        # No sense testing any further since we got the version number.
        exit(0);
      }
    }
  }
}
