#
# This script was written by George A. Theall, <theall@tifaware.com>.
#
# See the Nessus Scripts License for details.
#

if (description) {
  script_id(12647);
  script_version ("$Revision: 1.1 $");
 
  name["english"] = "SquirrelMail Detection";
  script_name(english:name["english"]);
 
  desc["english"] = "
This script detects whether the remote host is running SquirrelMail and
extracts its version number if so.  SquirrelMail is a PHP-based webmail
package that provides access to mail accounts via POP3 or IMAP; see
<http://www.squirrelmail.org/> for more information.

Risk factor : None";
  script_description(english:desc["english"]);
 
  summary["english"] = "Checks for the presence of SquirrelMail";
  script_summary(english:summary["english"]);
 
  script_category(ACT_GATHER_INFO);
  script_copyright(english:"This script is Copyright (C) 2004 George A. Theall");

  family["english"] = "General";
  script_family(english:family["english"]);

  script_dependencie("http_version.nasl");
  script_require_ports("Services/www", 80);

  exit(0);
}

include("global_settings.inc");
include("http_func.inc");
include("http_keepalive.inc");

port = get_http_port(default:80);
if (!get_port_state(port)) exit(0);

if (!can_host_php(port:port)) exit(0);

# Search for SquirrelMail in a couple of different locations.
dirs = make_list("", "/mail", "/sm", "/squirrelmail", "/webmail", cgi_dirs());
foreach d (dirs) {
  # Search in a couple of different pages.
  files = make_list(
    "/src/login.php", "/src/compose.php", "/ChangeLog", "/ReleaseNotes"
  );
  foreach f (files) {
    if (debug_level) display("debug: testing ", d, f, " on port ", port, "...\n");

    # Get the page.
    req = http_get(item:string(d, f), port:port);
    res = http_keepalive_send_recv(port:port, data:req);
    if (res == NULL) exit(0);           # can't connect
    if (debug_level) display("debug: res =>>", res, "<<\n");
    if (egrep(pattern:"200 OK", string:res, icase:TRUE)) {
      # Specify pattern used to identify version string.
      if (f == "/src/login.php" || f == "/src/compose.php") {
        pat = "<SMALL>SquirrelMail version (.+)<BR";
      }
      else if (f == "/ChangeLog") {
        pat = "^Version (.+) - [0-9]";
      }
      # nb: this first appeared in 1.2.0 and isn't always accurate.
      else if (f == "/ReleaseNotes") {
        pat = "Release Notes: SquirrelMail (.+) *\*";
      }
      # - someone updated files but forgot to add a pattern???
      else {
        display("Don't know how to handle file '", f, "'!\n");
        exit(1);
      }

      # Get the version string.
      if (debug_level) display("debug: grepping results for =>>", pat, "<<\n");
      matches = egrep(pattern:pat, string:res, icase:TRUE);
      foreach match (split(matches)) {
        match = chomp(match);
        if (debug_level) display("debug: grepping >>", match, "<< for =>>", pat, "<<\n");
        vers = eregmatch(pattern:pat, string:match, icase:TRUE);
        if (vers == NULL) break;

        if (debug_level) display("debug: SquirrelMail version =>>", vers[1], "<<\n");
        set_kb_item(name:string("www/", port, "/squirrelmail/version"), value:vers[1]);

        report = "The remote host is running SquirrelMail " + vers[1] + " under " + d + ".\n" +
                 "Squirrelmail is a PHP-based webmail package that provides access to\n" +
                 "mail accounts via POP3 or IMAP; see <http://www.squirrelmail.org/> for\n" +
                 "more information.\n";
        security_note(port:port, data:report);

        # No sense testing any further since we got the version number.
        exit(0);
      }
    }
  }
}
